'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = run;

var _colors = require('colors');

var _colors2 = _interopRequireDefault(_colors);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _replace = require('replace');

var _replace2 = _interopRequireDefault(_replace);

var _nodeFetch = require('node-fetch');

var _nodeFetch2 = _interopRequireDefault(_nodeFetch);

var _console2 = require('../../shared/console');

var _colors3 = require('./colors');

var _colors4 = _interopRequireDefault(_colors3);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const p = _path2.default.join;

function run(_ref) {
  let name = _ref.name;
  let use = _ref.use;
  let nocache = _ref.nocache;
  let debug = _ref.debug;

  try {
    // scaffolds as second argument
    // motion new app from/repo
    const scaffoldRepo = 'scaffold';
    let org = 'motion';
    let repo = scaffoldRepo;

    // scaffold
    if (use) {
      // third party scaffold
      if (use.indexOf('/') >= 0) {
        org = use.split('/')[0];
        repo = use.split('/')[1];
      }
      // official motion scaffold
      else {
          repo = `scaffold-${ use }`;
        }
    }

    // test for weird names
    if (/[^a-zA-Z0-9\_\-\$\.]/g.test(name)) {
      console.error('Name can\'t contain special characters'.bold.red);
      process.exit(1);
    }

    var spinner;
    let MOTION = {};
    MOTION.scaffoldDir = p(getUserHome(), '.motion', 'scaffold');
    MOTION.scaffoldRepo = `git://github.com/${ org }/${ repo }`;
    MOTION.scaffoldSHA = p(getUserHome(), '.motion', 'scaffoldSHA');
    MOTION.dest = p(process.cwd(), name);

    if (_fsExtra2.default.existsSync(MOTION.dest)) {
      console.log("Error! Directory %s already exists\n".red, MOTION.dest);
    } else {
      console.log();
      spinner = new _console2.Spinner('Creating app...  ');
      spinner.start();
      start();
    }

    function start() {
      makeFolder().then(nocache ? cloneDirectly : getScaffold).then(initGit).then(replaceGivenNameInApp).then(finish).then(() => {
        spinner.stop();
      }).catch(function (err) {
        spinner.stop();
        console.log("\n", 'Error'.bold.red);
        console.log(err);
      });
    }

    function makeFolder() {
      return new Promise(function (resolve, reject) {
        log('makeFolder', MOTION.dest);
        _fsExtra2.default.mkdirs(MOTION.scaffoldDir, function (err) {
          _fsExtra2.default.mkdirs(MOTION.dest, function (err) {
            if (err) reject(err);else resolve();
          });
        });
      });
    }

    function getScaffold() {
      return new Promise((res, rej) => {
        // check online
        promiseProcess('curl http://github.com', { msg: false }).then(() => {
          log('Online');
          next();
        }).catch(() => {
          log('Not online');
          copyScaffold().then(() => res());
        });

        var next = _fsExtra2.default.mkdirs.bind(null, MOTION.scaffoldDir, err => {
          log('fs.mkdirs err', err);
          const isCloningExample = repo != scaffoldRepo;

          if (err || isCloningExample) return cloneDirectly().then(() => res());

          return updateScaffoldCache().catch(cloneDirectly).then(doCopy => {
            log('doCopy?', doCopy);
            if (doCopy) return copyScaffold();else return cloneDirectly();
          }).then(() => {
            // continue
            log('CONTINUE FROM getScaffold');
            res();
          });
        });
      });
    }

    function updateScaffoldCache() {
      log('Looking for updated scaffold in %s', MOTION.scaffoldRepo);

      return new Promise(function (resolve, reject) {
        checkNewScaffold(needsNew => {
          log('checkNewScaffold needsNew?', needsNew);
          if (!needsNew) return resolve(true);

          // remove old scaffold
          _fsExtra2.default.remove(MOTION.scaffoldDir, function (err) {
            log('fs.remove err?', err);
            if (err) return resolve(false);

            // clone new scaffold
            promiseProcess(gitClone(MOTION.scaffoldDir), { msg: false }).then(() => copyLatestSHA(MOTION.scaffoldDir)).then(() => deleteGitFolder(MOTION.scaffoldDir)).then(() => resolve(true));
          });
        });
      });
    }

    function checkNewScaffold(cb) {
      log('Check for new scaffold SHA in...', MOTION.scaffoldSHA);
      _fsExtra2.default.readFile(MOTION.scaffoldSHA, function (err, data) {
        if (err) {
          log('Error reading scaffold file');
          return cb(true);
        }

        var local = data.toString().trim();
        log('Got ', local);

        (0, _nodeFetch2.default)('https://api.github.com/repos/motionjs/scaffold/commits').then(function (res) {
          return res.json();
        }).then(function (commits) {
          var remote = commits[0].sha.trim();
          log('Latest is', remote);

          if (local != remote) {
            message('Fetching latest scaffold...');
            // new
            cb(true);
          } else {
            // no new
            cb(false);
          }
        }).catch(err => {
          cb(true);
        });
      });
    }

    function deleteGitFolder(dir) {
      return new Promise(function (resolve, reject) {
        log('Remove .git folder');
        // delete git dir
        _fsExtra2.default.remove(p(dir, '/.git'), function (err) {
          log('after remove .git err?:', err);
          if (err) return reject(err);
          resolve();
        });
      });
    }

    function copyLatestSHA(dir) {
      return new Promise(function (res, rej) {
        log('copyLatestSHA');
        // copy latest SHA into folder
        var head = p(dir, '.git', 'refs', 'heads', 'master');
        log('Copy new SHA', head, MOTION.scaffoldSHA);

        try {
          var sha = _fsExtra2.default.readFileSync(head);
          _fsExtra2.default.writeFileSync(MOTION.scaffoldSHA, sha);
          res();
        } catch (e) {
          log('error with copyLatestSHA', e.stack);
          rej();
        }
      });
    }

    function copyScaffold() {
      log('Copy new scaffold', MOTION.scaffoldDir, MOTION.dest);
      return new Promise((res, rej) => {
        _fsExtra2.default.copy(MOTION.scaffoldDir, MOTION.dest, function (err) {
          if (err) {
            console.log("Error, couldn't copy scaffold folder".red);
            console.log(MOTION.scaffoldDir, MOTION.dest);
            process.exit(1);
          }

          res();
        });
      });
    }

    function gitClone(dest) {
      return 'git clone --depth=1 ' + MOTION.scaffoldRepo + ' ' + dest;
    }

    // clone right into target new folder
    function cloneDirectly() {
      log('Cloning directly', gitClone(MOTION.dest));
      return promiseProcess(gitClone(MOTION.dest), { msg: false }).then(() => deleteGitFolder(MOTION.dest));
    }

    function initGit() {
      log('Init new git');
      return promiseProcess('git init', { msg: false });
    }

    function replaceGivenNameInApp() {
      message('Setting app name...');
      return new Promise(function (resolve, reject) {
        log('Updating app name to', name, 'in', MOTION.dest);
        (0, _replace2.default)({
          regex: 'motion-scaffold',
          replacement: name,
          paths: [MOTION.dest],
          recursive: true,
          silent: true
        });

        // random color :)
        (0, _replace2.default)({
          regex: 'lightsalmon',
          replacement: (0, _colors4.default)(),
          paths: [p(MOTION.dest, 'main.js')],
          recursive: false,
          silent: true
        });

        resolve();
      });
    }

    function npmInstall() {
      log('Running npm install');
      return promiseProcess('npm install', {
        msg: 'Running npm install...',
        dir: MOTION.dest + '/.motion'
      });
    }

    function tryLinkMotion() {
      log('Try to link local motion');
      return promiseProcess('npm link motionjs', {
        msg: false,
        dir: MOTION.dest + '/.motion'
      });
    }

    const exec = require('child_process').exec;

    function promiseProcess(cmd, opts) {
      opts = opts || {};

      if (opts.msg !== false) message(opts.msg || cmd);

      return new Promise(function (resolve, reject) {
        process.chdir(opts.appDir || MOTION.dest);
        log(' $ ', cmd);
        exec(cmd, { uid: process.getuid() }, function () {
          for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          handleChildProcess.apply(undefined, [resolve, reject].concat(args));
        });
      });
    }

    function handleChildProcess(resolve, reject, error, stdout, stderr) {
      if (error) {
        log(stderr);
        return reject(error);
      }

      log(stdout);
      resolve();
    }

    function finish() {
      message('Done!');
      spinner.stop();
      console.log('  Done! ⇢'.green.bold, ` cd ${ name }\n`);
      process.exit();
    }

    function message(str) {
      spinner.message(str);
    }

    function log() {
      var _console;

      if (debug) (_console = console).log.apply(_console, arguments);
    }

    // mask (1 = execute, 4 = read, 2 = write)
    function checkPermission(file, mask, cb) {
      _fsExtra2.default.stat(file, function (error, stats) {
        if (error) {
          cb(error, false);
        } else {
          cb(null, !!(mask & parseInt((stats.mode & parseInt("777", 8)).toString(8)[0])));
        }
      });
    };

    function getUserHome() {
      return process.env[process.platform == 'win32' ? 'USERPROFILE' : 'HOME'];
    }
  } catch (e) {
    console.log(e.stack);
  }
}
//# sourceMappingURL=new.js.map